//=============================================================================
//   This file is part of VTKEdge. See vtkedge.org for more information.
//
//   Copyright (c) 2010 Kitware, Inc.
//
//   VTKEdge may be used under the terms of the BSD License
//   Please see the file Copyright.txt in the root directory of
//   VTKEdge for further information.
//
//   Alternatively, you may see: 
//
//   http://www.vtkedge.org/vtkedge/project/license.html
//
//
//   For custom extensions, consulting services, or training for
//   this or any other Kitware supported open source project, please
//   contact Kitware at sales@kitware.com.
//
//
//=============================================================================
#ifndef __vtkKWEPaintbrushData_h
#define __vtkKWEPaintbrushData_h

#include "VTKEdgeConfigure.h" // needed for export symbols directives
#include "vtkKWEPaintbrushEnums.h"
#include "vtkDataObject.h"

class vtkImageData;

class VTKEdge_WIDGETS_EXPORT vtkKWEPaintbrushData : public vtkDataObject
{
public:
  vtkTypeRevisionMacro(vtkKWEPaintbrushData, vtkDataObject);
  void PrintSelf(ostream& os, vtkIndent indent);

  // Description:
  // Minkowski operators. Return 1 if something changed.
  virtual int Add(      vtkKWEPaintbrushData *, bool forceMutable=false ) = 0;
  virtual int Subtract( vtkKWEPaintbrushData *, bool forceMutable=false ) = 0;
  virtual int Replace(  vtkKWEPaintbrushData *, bool forceMutable=false ) = 0;

  // Description:
  // Clip self with supplied extents. Return 1 if something changed
  virtual int Clip( int extent[6] ) = 0;

  // Description:
  // Set/Get extents.
  virtual void SetExtent( int extent[6] ) = 0;
  virtual void GetExtent( int extent[6] ) = 0;

  // Description:
  // Allocate the bulk data (once you've set the extents).
  virtual void Allocate( double fillValue = 0.0 ) = 0;

  // Description:
  // Metadata.
  virtual void SetSpacing(double spacing[3]) = 0;
  virtual void SetOrigin( double origin[3] ) = 0;

  // Description:
  // Is the point "p" inside ?
  virtual int IsInside( double p[3] ) = 0;

  // Description:
  // Get the brush data as an image data. Note that for grayscale brushes,
  // this is just a shallow copy of the data. For binary brushes, this
  // converts from the internal representation to an image data.
  virtual void GetPaintbrushDataAsImageData( vtkImageData * ) = 0;

  // Description:
  // Get the data type as an integer (this will return VTK_DATA_OBJECT
  // for now, maybe a proper type constant will be reserved later).
  int GetDataObjectType() { return VTK_DATA_OBJECT; }

  // Description:
  // The extent type is 3D, just like vtkImageData.
  int GetExtentType() { return VTK_3D_EXTENT; };

  // Description:
  // INTERNAL - Do not use.
  void SetLabel(vtkKWEPaintbrushEnums::LabelType l) { this->Label = l; }
  vtkGetMacro( Label, vtkKWEPaintbrushEnums::LabelType );

  // Description:
  // Clear data corresponding to a label.
  // - For vtkKWEPaintbrushStencilData, labels make no sense, since the data is
  //   binary. So   Clear(.) <==> Allocate()
  // - For vtkKWEPaintbrushGrayscaleData, labels make no sense. So
  //   Clear(.) <==> Allocate(0.0)
  // - For vtkKWEPaintbrushLabelData, this sets all the voxels with the supplied
  //   label value to 0.
  virtual void Clear( vtkKWEPaintbrushEnums::LabelType label );

protected:
  vtkKWEPaintbrushData();
  ~vtkKWEPaintbrushData();

  // Relevant only for vtkKWEPaintbrushStencilData when editing into label maps.
  vtkKWEPaintbrushEnums::LabelType Label;

private:
  vtkKWEPaintbrushData(const vtkKWEPaintbrushData&);  // Not implemented.
  void operator=(const vtkKWEPaintbrushData&);  // Not implemented.
};

#endif



